unit Generate;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, wcrypt2, Buttons;

type
  TGenerateForm = class(TForm)
    Label1: TLabel;
    ContainerEdit: TEdit;
    Label2: TLabel;
    KeyExchLenEdit: TEdit;
    UpDown1: TUpDown;
    Label3: TLabel;
    SignKeyLenEdit: TEdit;
    UpDown2: TUpDown;
    ReportMemo: TMemo;
    KEKCheckBox: TCheckBox;
    SKCheckBox: TCheckBox;
    OKBtn: TBitBtn;
    CloseBtn: TBitBtn;
    ClearBtn: TBitBtn;
    procedure OKBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  GenerateForm: TGenerateForm;

implementation

{$R *.DFM}

function algIDtostr(a: DWORD): string;
begin
case a of
CALG_HMAC: algIDtostr := 'HMAC keyed hash algorithm';
CALG_MD2: algIDtostr := 'MD2 hashing algorithm';
CALG_MD4: algIDtostr := 'MD4 hashing algorithm';
CALG_MD5: algIDtostr := 'MD5 hashing algorithm';
CALG_SHA: algIDtostr := 'SHA hashing algorithm';
CALG_MAC: algIDtostr := 'MAC keyed hash algorithm';
CALG_SSL3_SHAMD5: algIDtostr := 'SSL3 client authentication';
CALG_RSA_SIGN: algIDtostr := 'RSA public-key signature algorithm';
CALG_DSS_SIGN: algIDtostr := 'DSA public-key signature algorithm';
CALG_RSA_KEYX: algIDtostr := 'RSA public-key key exchange algorithm';
CALG_DES: algIDtostr := 'DES encryption algorithm';
CALG_RC2: algIDtostr := 'RC2 block encryption algorithm';
CALG_RC4: algIDtostr := 'RC4 stream encryption algorithm';
CALG_SEAL: algIDtostr := 'SEAL encryption algorithm';
CALG_DH_SF: algIDtostr := 'Diffie-Hellman store and forward key exchange algorithm';
else algIDtostr := 'unknown algorithm';
end;
end;

procedure TGenerateForm.OKBtnClick(Sender: TObject);
var cont: PChar;
    err: string;
    hProv: HCRYPTPROV;
    KeyExchKey, SignKey: HCRYPTKEY;
    flag, keyLen: DWORD;
begin
if not (KEKCheckBox.Checked or SKCheckBox.Checked) then exit;
if length(ContainerEdit.Text) = 0
then
    begin
    cont := nil;
    err := ' ';
    end
else
    begin
    err := ContainerEdit.Text;
    cont := StrAlloc(length(err) + 1);
    StrPCopy(cont, err);
    end;
if not CryptAcquireContext(@hProv, cont, nil, PROV_RSA_FULL, 0)
then
    begin
    case int64(GetLastError) of
    ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
    ERROR_NOT_ENOUGH_MEMORY: err := 'ERROR_NOT_ENOUGH_MEMORY';
    NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
    NTE_BAD_KEYSET: err := 'NTE_BAD_KEYSET';
    NTE_BAD_KEYSET_PARAM: err := 'NTE_BAD_KEYSET_PARAM';
    NTE_BAD_PROV_TYPE: err := 'NTE_BAD_PROV_TYPE';
    NTE_BAD_SIGNATURE: err := 'NTE_BAD_SIGNATURE';
    NTE_EXISTS: err := 'NTE_EXISTS';
    NTE_KEYSET_ENTRY_BAD: err := 'NTE_KEYSET_ENTRY_BAD';
    NTE_KEYSET_NOT_DEF: err := 'NTE_KEYSET_NOT_DEF';
    NTE_NO_MEMORY: err := 'NTE_NO_MEMORY';
    NTE_PROV_DLL_NOT_FOUND: err := 'NTE_PROV_DLL_NOT_FOUND';
    NTE_PROV_TYPE_ENTRY_BAD: err := 'NTE_PROV_TYPE_ENTRY_BAD';
    NTE_PROV_TYPE_NO_MATCH: err := 'NTE_PROV_TYPE_NO_MATCH';
    NTE_PROV_TYPE_NOT_DEF: err := 'NTE_PROV_TYPE_NOT_DEF';
    NTE_PROVIDER_DLL_FAIL: err := 'NTE_PROVIDER_DLL_FAIL';
    NTE_SIGNATURE_FILE_BAD: err := 'NTE_SIGNATURE_FILE_BAD';
    else err := 'Unknown error';
    end;
    MessageDlg('  : ' + err, mtError, [mbOK], 0);
    exit;
    end;
if KEKCheckBox.Checked then
   begin
   keyLen := strtoint(KeyExchLenEdit.text);
   flag := keyLen shl 16;
   if not CryptGenKey(hProv, AT_KEYEXCHANGE, flag, @KeyExchKey) then
      begin
      case int64(GetLastError) of
      ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
      ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
      NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
      NTE_BAD_ALGID: err := 'NTE_BAD_ALGID';
      NTE_BAD_UID: err := 'NTE_BAD_UID';
      NTE_FAIL: err := 'NTE_FAIL';
      else err := 'Unknown error';
      end;
      MessageDlg('    : ' + err,
                         mtError, [mbOK], 0);
      end
   else
       begin
       ReportMemo.Lines.Add('');
       ReportMemo.Lines.Add('   :');
       flag := 4;
       if not CryptGetKeyParam(KeyExchKey, KP_KEYLEN, @keyLen, @flag, 0) then
          begin
          case int64(GetLastError) of
          ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
          ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
          ERROR_MORE_DATA: err := 'ERROR_MORE_DATA';
          NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
          NTE_BAD_KEY: err := 'NTE_BAD_KEY';
          NTE_NO_KEY: err := 'NTE_NO_KEY';
          NTE_BAD_TYPE: err := 'NTE_BAD_TYPE';
          NTE_BAD_UID: err := 'NTE_BAD_UID';
          else err := 'Unknown error';
          end;
          MessageDlg('   : ' + err,
                          mtError, [mbOK], 0);
          end
       else ReportMemo.Lines.Add('    - ' + inttostr(keyLen));
       flag := 4;
       if not CryptGetKeyParam(KeyExchKey, KP_ALGID, @keyLen, @flag, 0) then
          begin
          case int64(GetLastError) of
          ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
          ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
          ERROR_MORE_DATA: err := 'ERROR_MORE_DATA';
          NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
          NTE_BAD_KEY: err := 'NTE_BAD_KEY';
          NTE_NO_KEY: err := 'NTE_NO_KEY';
          NTE_BAD_TYPE: err := 'NTE_BAD_TYPE';
          NTE_BAD_UID: err := 'NTE_BAD_UID';
          else err := 'Unknown error';
          end;
          MessageDlg('   : ' + err,
                          mtError, [mbOK], 0);
          end
       else ReportMemo.Lines.Add('   - ' + algIDtostr(keyLen));
       end;
   end;
if SKCheckBox.Checked then
   begin
   keyLen := strtoint(SignKeyLenEdit.text);
   flag := keyLen shl 16;
   if not CryptGenKey(hProv, AT_SIGNATURE, flag, @SignKey) then
      begin
      case int64(GetLastError) of
      ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
      ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
      NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
      NTE_BAD_ALGID: err := 'NTE_BAD_ALGID';
      NTE_BAD_UID: err := 'NTE_BAD_UID';
      NTE_FAIL: err := 'NTE_FAIL';
      else err := 'Unknown error';
      end;
      MessageDlg('   : ' + err,
                         mtError, [mbOK], 0);
      end
   else
       begin
       ReportMemo.Lines.Add('');
       ReportMemo.Lines.Add('  :');
       flag := 4;
       if not CryptGetKeyParam(SignKey, KP_KEYLEN, @keyLen, @flag, 0) then
          begin
          case int64(GetLastError) of
          ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
          ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
          ERROR_MORE_DATA: err := 'ERROR_MORE_DATA';
          NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
          NTE_BAD_KEY: err := 'NTE_BAD_KEY';
          NTE_NO_KEY: err := 'NTE_NO_KEY';
          NTE_BAD_TYPE: err := 'NTE_BAD_TYPE';
          NTE_BAD_UID: err := 'NTE_BAD_UID';
          else err := 'Unknown error';
          end;
          MessageDlg('   : ' + err,
                          mtError, [mbOK], 0);
          end
       else ReportMemo.Lines.Add('    - ' + inttostr(keyLen));
       flag := 4;
       if not CryptGetKeyParam(SignKey, KP_ALGID, @keyLen, @flag, 0) then
          begin
          case int64(GetLastError) of
          ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
          ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
          ERROR_MORE_DATA: err := 'ERROR_MORE_DATA';
          NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
          NTE_BAD_KEY: err := 'NTE_BAD_KEY';
          NTE_NO_KEY: err := 'NTE_NO_KEY';
          NTE_BAD_TYPE: err := 'NTE_BAD_TYPE';
          NTE_BAD_UID: err := 'NTE_BAD_UID';
          else err := 'Unknown error';
          end;
          MessageDlg('   : ' + err,
                          mtError, [mbOK], 0);
          end
       else ReportMemo.Lines.Add('   - ' + algIDtostr(keyLen));
       end;
   end;
if not CryptReleaseContext(hProv, 0) then
   begin
   case int64(GetLastError) of
   ERROR_BUSY: err := 'ERROR_BUSY';
   ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
   ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
   NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
   NTE_BAD_UID: err := 'NTE_BAD_UID';
   else err := 'Unknown error';
   end;
   MessageDlg('   : ' + err,
       mtError, [mbOK], 0);
   end;
end;

procedure TGenerateForm.ClearBtnClick(Sender: TObject);
begin
ReportMemo.Lines.Clear;
end;

procedure TGenerateForm.CloseBtnClick(Sender: TObject);
begin
Close;
end;

end.
